/*
Freely inspired on http://cortediazathoth.altervista.org/?eye code !

Copyright (C) 2013  jul@siilia.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

const Main = imports.ui.main;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Panel = imports.ui.panel;
const St = imports.gi.St;
const PanelMenu = imports.ui.panelMenu;

// aka wait between two samples
const UPDATE_INTERVAL = 200;
// size in mm of one of your monitor pixel
const PIXEL_SIZE = 0.29;

const MtrPosition = {
    CENTER: 0,
    RIGHT: 1,
    LEFT: 2
}

// global instance object
let mtr;
 

/* Entry points ****************************/
function main() {
    init();
    enable();
}
function init() { }
function enable() {
    mtr = new Mtr(MtrPosition.CENTER, 0);
    mtr.setActive(true);
    Main.panel.addToStatusArea('mtr', mtr);
}
 
function disable() {
    mtr.setActive(false);
    mtr.destroy();
}
/*******************************************/

 
function Mtr( position_in_panel, index_in_panel) {
    this._init( position_in_panel, index_in_panel);
}
 
Mtr.prototype = {
    __proto__: PanelMenu.Button.prototype,
 
    _init: function( position_in_panel, index_in_panel) {
        PanelMenu.Button.prototype._init.call(this, St.Align.START);
 
       	// load ressources...
				this.icon = new St.Icon({ icon_name: 'input-mouse-symbolic', style_class: 'popup-menu-icon ' });
        this.info = new St.Label({text: "--"});  
				
				// format them...
				let topBox = new St.BoxLayout();
				topBox.add_actor(this.icon);
				topBox.add_actor(this.info);
				this.actor.add_actor(topBox);
				
				// init stuff
        this.position_in_panel = position_in_panel;
 				let [mouse_x, mouse_y, mask] = global.get_pointer();
 				this.ox = mouse_x;
 				this.oy = mouse_y;
 				this.l = 0;
 				this.lmm = 0;

        let children = null;
        let box = null;
        switch (this.position_in_panel) {
            case MtrPosition.LEFT:
                box = Main.panel._leftBox;
                break;
            case MtrPosition.CENTER:
                box = Main.panel._centerBox;
                break;
            case MtrPosition.RIGHT:
                box = Main.panel._rightBox;
                break;
        }
        children = box.get_children();
        if(index_in_panel < 0)
            index_in_panel = children.length +1 +index_in_panel;
         
        box.insert_child_at_index(this.actor, index_in_panel);
    },
 
    setActive: function(enabled) {
        
        if (enabled) {
            this._update_handler = Mainloop.timeout_add(UPDATE_INTERVAL, Lang.bind(this, this._on_timeout));
        } else {
            if(this._update_handler)
	            Mainloop.source_remove(this._update_handler);
        }
    }, 
 
 		getDistance: function() {
 			if( this.lmm < 1000 )
 				return Math.round(this.lmm)+"mm";
 			if( this.lmm < 1000*1000 )
 				return (Math.round((this.lmm/1000)*10)/10)+"m";

			return (Math.round(this.lmm/100000)*10)/100+"km";
 		},
 
    _on_timeout: function() {
        // get mouse position
        let [mouse_x, mouse_y, mask] = global.get_pointer();
				// First pass.. no move
				if( this.ox == 0 ) { this.ox = mouse_x; return true; }
				// If no move since last call, quit.
				if( mouse_x == this.ox && mouse_y == this.oy ) return true;
				
				// compute distance
				tx = Math.abs((mouse_x-this.ox));
				ty = Math.abs((mouse_y-this.oy));
        this.l += Math.round(Math.sqrt((tx*tx)+(ty*ty)));
        this.lmm = this.l * PIXEL_SIZE;
				// update label
        this.info.set_text( this.getDistance() );
				// save for next timeout...
        this.ox = mouse_x;
        this.oy = mouse_y;
        return true;
    },
}
